// -*- mode:c++ -*-
//
// Module AddressHeap.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: AddressHeap.cc,v 1.2 2002/09/02 18:23:33 raoul Exp $
//

#include "AddressHeap.hh"
#include "PackageGroup.hh"

void resetHeap (HeapType &heap
		, AddressSet const &byAddress
		, SpanningTree const &tree
		, StaticBoard const &board
		, int &cap)
{
  AddressSet::const_iterator iter = byAddress.begin();

  heap.clear();

  while (iter != byAddress.end())
    {
      int x = iter->mDestX;
      int y = iter->mDestY;
      int location = board.offset (x, y);

      AddressInfo info (cap, location, tree.distance (location));

      info += *iter;

      ++iter;

      while ((iter != byAddress.end())
	     && (iter->mDestX == x)
	     && (iter->mDestY == y))
	{
	  info += *iter;
	  ++iter;
	}

      heap.push_back (info);
    }

  recalcHeap (heap);
}

void recalcHeap (HeapType &heap)
{
  std::make_heap (heap.begin()
		  , heap.end()
		  , greaterMetric<AddressInfo const>);
}

void resetCapacityRef (HeapType &heap, int &cap)
{
  for (HeapType::iterator iter = heap.begin()
	 ; iter != heap.end()
	 ; ++iter)
    {
      iter->mCapacityPtr = &cap;
    }
}

void resetDistance (HeapType &heap, SpanningTree const &tree)
{
  for (HeapType::iterator iter = heap.begin()
	 ; iter != heap.end()
	 ; ++iter)
    {
      iter->mDistance = tree.distance (iter->mLocation);
    }
}
