// -*- mode:c++ -*-
//
// Header file AddressInfo.hh
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: AddressInfo.hh,v 1.2 2002/09/02 18:23:40 raoul Exp $
//

#ifndef AddressInfo_rmg_20020902_included
#define AddressInfo_rmg_20020902_included

#include <iostream>

struct AddressInfo
{
  AddressInfo (int location) : mLocation (location) { }

  AddressInfo (int const &capacity, int location, int d)
    : mCapacityPtr (&capacity)
    , mLocation (location)
    , mDistance (d)
    , mWeight (0)
    , mPackages (0)
  {
  }

  void operator+= (PackageInfo const &info)
  {
    ++mPackages;
    mWeight += info.mWeight;
  }

  double metric () const
  {
    double meanWeight = mWeight / (double) mPackages;

    int n = (int) ((*mCapacityPtr) / meanWeight); // Round down

    return n * mWeight / mDistance;
  }

  int const *mCapacityPtr;
  int mLocation;
  double mDistance;

  int mWeight;
  int mPackages;
};

template<typename T>
inline bool greaterMetric (T &t1, T &t2)
{
  return t1.metric() > t2.metric();
}

inline std::ostream &operator<< (std::ostream &strm, AddressInfo const &info)
{
  strm
    << "l"
    << info.mLocation
    << " d"
    << info.mDistance
    << " n"
    << info.mPackages
    << " w"
    << info.mWeight
    << " m"
    << info.metric();
}

#endif // AddressInfo_rmg_20020902_included
