// -*- mode:c++ -*-
//
// Module PackageGroup.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: PackageGroup.cc,v 1.1 2002/09/02 16:34:12 raoul Exp raoul $
//

#include "PackageGroup.hh"

PackageGroup::PackageGroup () : mTotalWeight (0), mCount (0) { }

void PackageGroup::insert (PackageInfo const &p)
{
  mById.insert (std::make_pair (p.mId, p));
  mByAddress.insert (p);
  mTotalWeight += p.mWeight;
  ++mCount;
}

PackageInfo PackageGroup::remove (int id)
{
  if (mById.find (id) == mById.end())
    {
      return PackageInfo (id);
    }

  PackageInfo result = mById[id];

  mTotalWeight -= result.mWeight;
  --mCount;

  mById.erase (id);
  mByAddress.erase (result);

  return result;
}

void PackageGroup::reset ()
{
  mTotalWeight = mCount = 0;
  mById.clear();
  mByAddress.clear();
}

AddressSet::const_iterator
PackageGroup::beginAt (std::pair<int, int> const &loc) const
{
  PackageInfo min (0);
  min.mDestX = loc.first;
  min.mDestY = loc.second;
  min.mWeight = 0;

  return mByAddress.lower_bound (min);
}

AddressSet::const_iterator
PackageGroup::endAt (std::pair<int, int> const &loc) const
{
  PackageInfo max (0);
  max.mDestX = loc.first;
  max.mDestY = loc.second + 1;
  max.mWeight = 0;

  return mByAddress.lower_bound (max);
}
