// -*- mode:c++ -*-
//
// Module Robot.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: Robot.cc,v 1.3 2002/09/02 16:34:37 raoul Exp raoul $
//

#include "Robot.hh"
#include <cassert>
#include "World.hh"

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

Robot::Robot (World &world, int id)
  : mWorldPtr (&world)
  , mLocation (-1)
  , mId (id)
  , mCapacity (0)
  , mMoney (0)
  , mScore (0)
{
}

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

Robot::Robot (World &world, RobotInfo const &info)
  : mWorldPtr (&world)
  , mLocation (-1)
  , mId (info.mId)
  , mCapacity (info.mCapacity)
  , mMoney (info.mMoney)
{
}

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

Robot::~Robot ()
{
}

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

void Robot::setLocation (int loc)
{
  mLocation = loc;
}

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

void Robot::pickPackage (PackageInfo const &package)
{
  mPackages.insert (package);
}

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

PackageInfo Robot::dropPackage (int id)
{
  PackageInfo info = mPackages.remove (id);

  if (info.mDestX == -1)
    {
      // We didn't know about this package, but it has to be
      // at our location, since we're dropping it :-)

      info.mDestX = worldPtr()->board().whereIs (location()).first;
      info.mDestX = worldPtr()->board().whereIs (location()).second;
    }

  return info;
}

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////

int Robot::location ()
{
  return mLocation;
}

/////////////////////////////////////////////////////////////////////////////
// Capacity
/////////////////////////////////////////////////////////////////////////////

int Robot::capacity () const
{
  return mCapacity;
}

/////////////////////////////////////////////////////////////////////////////
// Money
/////////////////////////////////////////////////////////////////////////////

int Robot::money () const
{
  return mMoney; // Compensate for passage of time?
}
