// -*- mode:c++ -*-
//
// Module Searcher.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: Searcher.cc,v 1.3 2002/09/02 12:10:31 raoul Exp $
//

#include "Searcher.hh"
#include "World.hh"
#include <boost/graph/breadth_first_search.hpp>

Searcher::Action Searcher::Limiter::action (TinyEdgeSet::Edge const &
					    , World const &) const
{
  return proceed;
}

Searcher::Action
Searcher::AnyBaseLimiter::action (TinyEdgeSet::Edge const &edge
				  , World const &world) const
{
  if (world.board().isBase (edge.second))
    {
      *mNearestBasePtr = edge.second;
      return cancel;
    }

  else
    {
      return proceed;
    }
}

Searcher::Action
Searcher::TargetSquareLimiter::action (TinyEdgeSet::Edge const &edge
				       , World const &) const
{
  return edge.second == mTarget ? cancel : proceed;
}

namespace
{
  struct Visitor : public boost::default_bfs_visitor
  {
    Visitor (std::vector<boost::default_color_type> &colours
	     , World const &world
	     , SpanningTree &tree
	     , Searcher::Limiter const &limiter)

      : mColourPtr (&colours)
      , mWorldPtr (&world)
      , mTreePtr (&tree)
      , mLimiterPtr (&limiter)
    {
    }

    void tree_edge (TinyEdgeSet::Edge const &edge, const TESGraph &)
    {
      debugStream() << edge.first << "," << edge.second << " ";

      mTreePtr->setPredecessor (edge.second, edge.first);

      Searcher::Action act = mLimiterPtr->action (edge, *mWorldPtr);

      if (act == Searcher::prune)
	{
	  (*mColourPtr)[edge.second]
	    = boost::color_traits<std::vector<boost::default_color_type>::value_type>::black();
	}

      else if (act == Searcher::cancel)
	{
	  std::fill (mColourPtr->begin()
		     , mColourPtr->end()
		     , boost::color_traits<std::vector<boost::default_color_type>::value_type>::black());
	}
    }

  private:
    std::vector<boost::default_color_type> *mColourPtr;
    World const *mWorldPtr;
    SpanningTree *mTreePtr;
    Searcher::Limiter const *mLimiterPtr;
  };
}

Searcher::Searcher (World const &world)
  : mWorldPtr (&world)
  , mGraph (world.board().makeGraph())
  , mColours (mGraph.vertices()) 
{
}

int Searcher::treeSize () const
{
  return mGraph.vertices();
}

void Searcher::search (SpanningTree &tree)
{
  search (tree, Limiter());
}

void Searcher::search (SpanningTree &tree, Limiter const &limiter)
{
  using namespace boost;

  typedef iterator_property_map <std::vector<default_color_type>::iterator
    , identity_property_map
    , default_color_type
    , default_color_type &> PropMap;

  breadth_first_search
    (mGraph
     , tree.root()
     , visitor (Visitor (mColours, *mWorldPtr, tree, limiter))
     . color_map (PropMap (mColours.begin()))
     . vertex_index_map (identity_property_map()));
}

