// -*- mode:c++ -*-
//
// Header file Searcher.hh
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: Searcher.hh,v 1.2 2002/09/02 12:07:45 raoul Exp $
//

#ifndef Searcher_rmg_20020902_included
#define Searcher_rmg_20020902_included

#include <vector>
//#include <boost/graph/properties.hpp>

#include "TESGraph.hh"

class SpanningTree;
class World;

class Searcher
{
  typedef std::vector<boost::default_color_type> ColourType;

public:
  Searcher (World const &);

  void search (SpanningTree &);

  int treeSize () const;

public:
  typedef enum {
    proceed
    , prune
    , cancel
  } Action;

  struct Limiter
  {
    // Base class returns proceed in all cases
    virtual Action action (TinyEdgeSet::Edge const &
			   , World const &) const;
  };

  void search (SpanningTree &, Limiter const &);

  struct AnyBaseLimiter : public Limiter
  {
    AnyBaseLimiter (int *nearest) : mNearestBasePtr (nearest) { }

    Action action (TinyEdgeSet::Edge const &, World const &) const;

    int *mNearestBasePtr;
  };

  struct TargetSquareLimiter : public Limiter
  {
    TargetSquareLimiter (int target) : mTarget (target) { }

    Action action (TinyEdgeSet::Edge const &, World const &) const;

    int mTarget;
  };

private:
  World const *mWorldPtr;
  TESGraph mGraph;
  std::vector<boost::default_color_type> mColours;
};

#endif // Searcher_rmg_20020902_included
