// -*- mode:c++ -*-
//
// Module ServerConnection.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 8/31	rmg	File creation
//
// $Id: ServerConnection.cc,v 1.1 2002/08/31 16:30:44 raoul Exp $
//

#include "ServerConnection.hh"

#include <iostream>
#include <cstdlib>
#include <algorithm>
#include <string>
#include <iterator>
#include <string>
#include <stdexcept>

#include "socket_port.hh"
#include "fdstream.hpp"

////////////////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////////////////

ServerConnection::ServerConnection (int argc, char *argv[])
{
  if (argc < 3)
    {
      std::cerr << "Usage: client host port\n";
      throw std::runtime_error ("Connection failed");
    }

  OS_STARTUP;

  int mSocketId = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);

  sockaddr_in serverAddr;
  serverAddr.sin_family = AF_INET;
  serverAddr.sin_port = htons (atoi (argv[2]));
  serverAddr.sin_addr.s_addr = inet_addr (argv[1]);

  struct hostent *hEnt = 0;

  if (serverAddr.sin_addr.s_addr != INADDR_NONE)
    {
      hEnt = gethostbyaddr ((char *) &serverAddr.sin_addr.s_addr
			    , sizeof (serverAddr.sin_addr.s_addr)
			    , AF_INET);
    }

  else
    {
      hEnt = gethostbyname (argv[1]);
    }

  if (hEnt == 0)
    {
      std::cerr 
	<< "Unable to resolve host " << argv[1]
	<< " ("
	<< LASTERROR
	<< ")\n";

      throw std::runtime_error ("Connection failed");
    }

  int count = 0;
  bool connected = false;

  while (hEnt->h_addr_list[count] && !connected)
    {
      serverAddr.sin_addr = *(in_addr *) hEnt->h_addr_list[count];

      if (-1 != connect (mSocketId
			 , (sockaddr *) &serverAddr
			 , sizeof (serverAddr)))
	{
	  connected = true;
	}

      else
	{
	  std::cerr
	    << "Unable to connect to "
	    << inet_ntoa (serverAddr.sin_addr)
	    << " "
	    << ntohs (serverAddr.sin_port)
	    << " ("
	    << LASTERROR
	    << ")\n";

	  ++count;
	}
    }

  if (!connected)
    {
      std::cerr << "Exhausted address list. Aborting\n";

      throw std::runtime_error ("Connection failed");
    }

  struct linger l = { 1, 0 };

  if (-1 == setsockopt (mSocketId
			, SOL_SOCKET
			, SO_LINGER
			, (char *) &l
			, sizeof(l)))
    {
      std::cerr << "Warning: setsockopt failed\n";
    }

  mInPtr = new boost::fdistream (mSocketId);
  mOutPtr = new boost::fdostream (mSocketId);
}

////////////////////////////////////////////////////////////////////////////
// Destructor
////////////////////////////////////////////////////////////////////////////

ServerConnection::~ServerConnection ()
{
  delete mInPtr;
  delete mOutPtr;
  CLOSESOCKET (mSocketId);

  OS_SHUTDOWN;
}

////////////////////////////////////////////////////////////////////////////
// Accessors (here because they need full fdstream definitions)
////////////////////////////////////////////////////////////////////////////

std::istream *ServerConnection::inPtr() const { return mInPtr; }
std::ostream *ServerConnection::outPtr() const { return mOutPtr; }
