// -*- mode:c++ -*-
//
// Header file Sorter.hh
//
// Copyright (c) 2002 Raoul M. Gough
//
// Attempt to find the best ordering of list of items according to
// some metric function, but limit the number of orderings considered.
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: Sorter.hh,v 1.1 2002/09/02 10:01:39 raoul Exp $
//

#ifndef Sorter_rmg_20020902_included
#define Sorter_rmg_20020902_included

#include <stdlib.h> // Or wherever random () defined is

template<typename CONTAINER, typename METRIC>
typename METRIC::result_type
optimizeOrdering (CONTAINER &cont
		  , METRIC &metric
		  , int limit)
{
  if (cont.size() < 2)
    {
      return metric (cont);
    }

  CONTAINER copy (cont);

  typename METRIC::result_type bestSoFar = metric (copy);

  while (limit--)
    {
      CONTAINER::iterator prev = copy.begin();
      CONTAINER::iterator iter = copy.begin();

      // Randomize the container!
      do
	{
	  ++iter;

	  if (random() & 64) // Check any old bit to get 50%
	    {
	      std::swap (*prev, *iter);
	    }

	  prev = iter;

	} while (iter != copy.end());

      typename METRIC::result_type temp = metric (copy);

      if (temp > bestSoFar)
	{
	  cont = copy;
	  bestSoFar = temp;
	}
    }

  return bestSoFar;
}

#endif // Sorter_rmg_20020902_included
