// -*- mode:c++ -*-
//
// Header file SpanningTree.hh
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 1	rmg	File creation
//
// $Id: SpanningTree.hh,v 1.4 2002/09/02 16:34:34 raoul Exp $
//

#ifndef SpanningTree_rmg_20020901_included
#define SpanningTree_rmg_20020901_included

#include <vector>
#include "Path.hh"

class SpanningTree
{
public:
  SpanningTree (int nVertices, int root);

  int root () const { return mRoot; }

  bool onTree (int target) const { return mPredecessor[target] != -1; }
  // Check if we have a route between root to target (if we do, it is
  // the shortest route)

  int nextToRoot (int from) const { return mPredecessor[from]; }
  // Returns the next vertex on the shortest path from the given
  // vertex to the root. Returns -1 if no route exists.

  Path pathFromRoot (int target) const;
  // Useful if you want to discard the rest of the tree and keep
  // a small number of routes.
  // Returns an empty container if target is equal to root
  // *or* if there is no route between root and target in the tree.

  int distance (int target) const
  {
    if (!onTree (target))
      {
	return INT_MAX;
      }

    // Could be faster
    return pathFromRoot (target).totalLength();
  }

public:
  void reset (int root);
  void setPredecessor (int of, int pred) { mPredecessor[of] = pred; }

private:
  std::vector<int> mPredecessor;
  int mRoot;
};

#endif // SpanningTree_rmg_20020901_included
