// -*- mode:c++ -*-
//
// Header file TinyEdgeIterator.hh
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 1	rmg	File creation
//
// $Id: TinyEdgeIterator.hh,v 1.4 2002/09/01 21:04:31 raoul Exp $
//

#ifndef TinyEdgeIterator_rmg_20020901_included
#define TinyEdgeIterator_rmg_20020901_included

#include "TinyEdgeSet.hh"
#include <iterator>

#include "LogStream.hh"

class TinyEdgeIterator
  : public std::forward_iterator<TinyEdgeSet::Edge, size_t>
{
public:
  TinyEdgeIterator (TinyEdgeSet const *setPtr
		    , TinyEdgeSet const *limitPtr
		    , int n
		    , int width)
    : mEdgeSetPtr (setPtr)
    , mLimitPtr (limitPtr)
    , mVertex (n)
    , mWidth (width)
    , mDir (TinyEdgeSet::dir_base)
  {
    debugStream() << "tei (" << setPtr << ", " << limitPtr << ")\n";

    if ((setPtr != limitPtr) && (!mEdgeSetPtr->hasEdge (mDir)))
      {
	operator++();
      }
  }

  TinyEdgeIterator ()
    : mEdgeSetPtr (0)
    , mVertex (0)
    , mWidth (0)
  {
    debugStream() << "tei ()\n";
  }

  TinyEdgeSet::Edge operator* ()
  {
    debugStream() << "tei: edge " << mVertex << ":" << mDir << "\n";

    return TinyEdgeSet::Edge (mVertex, mWidth, mDir);
  }

  TinyEdgeIterator &operator++ ()
  {
    //
    // Scan for new edge in this set. Stop if limit reached
    //
    do
      {
	mDir = static_cast<TinyEdgeSet::Direction>(static_cast<int>(mDir) + 1);

	if (mDir == TinyEdgeSet::dir_limit)
	  {
	    ++mEdgeSetPtr;
	    ++mVertex;
	    mDir = TinyEdgeSet::dir_base;
	  }
      }
    while ((mEdgeSetPtr < mLimitPtr) && (!mEdgeSetPtr->hasEdge (mDir)));

    debugStream() << "tei++ at " << mEdgeSetPtr << " of " << mLimitPtr << "\n";

    return *this;
  }

  TinyEdgeIterator operator++ (int)
  {
    TinyEdgeIterator temp = *this;
    ++(*this);
    return temp;
  }

  bool operator== (TinyEdgeIterator const &other)
  {
    bool result
      = (mEdgeSetPtr == other.mEdgeSetPtr)
      && (mVertex == other.mVertex)
      && (mWidth == other.mWidth)
      && (mDir == other.mDir);

    debugStream() << "tei at " << mEdgeSetPtr << ":" << mDir;

    if (result)
      {
	debugStream() << " == ";
      }

    else
      {
	debugStream() << " != ";
      }

    debugStream() << other.mEdgeSetPtr << ":" << other.mDir << "\n";

    return result;
  }

  bool operator!= (TinyEdgeIterator const &other)
  {
    return !((*this) == other);
  }

private:
  TinyEdgeSet const *mEdgeSetPtr;
  TinyEdgeSet const *mLimitPtr;
  int mWidth;
  int mVertex;
  TinyEdgeSet::Direction mDir;
};

#endif // TinyEdgeIterator_rmg_20020901_included
