// -*- mode:c++ -*-
//
// Module runrobotrun.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 2	rmg	File creation
//
// $Id: runrobotrun.cc,v 1.3 2002/09/02 17:38:34 raoul Exp $
//

#include "World.hh"
#include "LogStream.hh"
#include "ServerConnection.hh"

#include <stdexcept>

namespace
{
  void runrunrun (std::istream &, std::ostream &);
}

int main (int argc, char *argv[])
{
  int xStart = -1;
  int yStart = -1;

  if (argc > 4)
    {
      yStart = atoi (argv[--argc]);
      xStart = atoi (argv[--argc]);
    }

  if ((argc == 4) || (argc == 2))
    {
      --argc;

      switch (atoi (argv[argc]))
	{
	case 9:
	  debugStream (&std::cerr);
	  debugStream() << "Begin debug log\n";
	  // Fall through

	case 2:
	  commStream (&std::cerr);
	  commStream() << "Begin communication log\n";
	  // Fall through

	default:
	  logStream (&std::cerr);
	  logStream() << "Begin main log\n";
	  break;
	}
    }

  try
    {
      if (argc == 1)
	{
	  std::cerr
	    << "Using stdin/out."
	    " To connect to a server, use: runrobotrun host port [dbg [x y]]\n";

	  runrunrun (std::cin, std::cout);
	}

      else
	{
	  std::cerr << "Attempting server connection\n";

	  ServerConnection conn (argc, argv);

	  if (xStart != -1)
	    {
	      *conn.outPtr()
		<< "Login "
		<< xStart + 1
		<< " "
		<< yStart + 1
		<< std::endl;
	    }

	  else
	    {
	      *conn.outPtr() << "Player" << std::endl;
	    }

	  std::cerr << "Connected\n";

	  runrunrun (*conn.inPtr(), *conn.outPtr());
	}
    }

  catch (std::runtime_error const &err)
    {
      std::cerr << "Caught error: " << err.what() << std::endl;
    }
}

namespace
{
  void runrunrun (std::istream &istrm, std::ostream &ostrm)
  {
    World world (istrm);

    logStream() << world.board() << std::endl;

    while ((!istrm.eof()) && (world.smartRobot()))
      {
	istrm.clear(); // Ignore all stream errors and hope for the best :-)

	world.smartRobot()->makeMove (ostrm);
	world.update (istrm);
      }

    if (istrm.eof())
      {
	std::cerr << "End of input. ";
      }

    else
      {
	std::cerr << "Smart robot has died. ";
      }

    std::cerr << "Game over\n";
  }
}
