// -*- mode:c++ -*-
//
// Module server.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 8/30	rmg	File creation
//
// $Id: server.cc,v 1.5 2002/08/31 10:49:07 raoul Exp $
//

#include <iostream>
#include <cstdlib>
#include <algorithm>
#include <string>
#include <iterator>

#include "socket_port.hh"

#include "fdstream.hpp"
#include <boost/smart_ptr.hpp>

int main ()
{
  OS_STARTUP;

  int const maxBacklog = 5;
  int const serverPort = 7453;

  sockaddr_in listenaddr;
  listenaddr.sin_family = AF_INET;
  listenaddr.sin_port = htons (serverPort);
  listenaddr.sin_addr.s_addr = INADDR_ANY;
  memset (listenaddr.sin_zero, 0, sizeof (listenaddr.sin_zero));

  int listener = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);

  if (listener == -1)
    {
      std::cerr << "socket: " << LASTERROR << "\n";
    }

  else if (bind (listener
		 , (sockaddr *) &listenaddr
		 , sizeof (listenaddr)) == -1)
    {
      std::cerr << "bind: " << LASTERROR << "\n";
    }

  else if (listen (listener, maxBacklog) == -1)
    {
      std::cerr << "listen: " << LASTERROR << "\n";
    }

  else
    {
      sockaddr_in connaddr;
      accept_len_type len = sizeof (connaddr);

      std::cerr << "Listening on port " << ntohs (listenaddr.sin_port) << "\n";

      int connSock = accept (listener, (sockaddr *) &connaddr, &len);

      if (connSock == -1)
	{
	  std::cerr << "accept: " << LASTERROR << "\n";
	  return 1;
	}

      else
	{
	  std::cerr
	    << "Connection from "
	    << inet_ntoa (connaddr.sin_addr)
	    << " "
	    << ntohs (connaddr.sin_port)
	    << "\n";

	  struct linger l = { 1, 0 };

	  if (-1 == setsockopt (connSock
				, SOL_SOCKET
				, SO_LINGER
				, (char *) &l
				, sizeof(l)))
	    {
	      std::cerr << "Warning: setsockopt failed\n";
	    }

	  boost::fdistream conn_istrm (connSock);

	  std::copy (std::istream_iterator<std::string> (conn_istrm)
		     , std::istream_iterator<std::string> ()
		     , std::ostream_iterator<std::string> (std::cerr, "\n"));
	}

      if (CLOSESOCKET (connSock) == -1)
	{
	  std::cerr << "closesocket: " << LASTERROR << "\n";
	}
    }

  OS_SHUTDOWN;

  return 0;
}
