// -*- mode:c++ -*-
//
// Module testgraph.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 9/ 1	rmg	File creation
//
// $Id: testgraph.cc,v 1.12 2002/09/02 16:35:27 raoul Exp $
//

#include "Searcher.hh"

#include <iostream>
#include <typeinfo>
#include <iomanip>
#include <stdexcept>
#include <vector>
#include <deque>
#include <algorithm>
#include <iterator>
#include <boost/graph/breadth_first_search.hpp>

#include "TESGraph.hh"
#include "World.hh"
#include "LogStream.hh"
#include "SpanningTree.hh"

namespace
{
  void runTest ();
}

int main (int argc, char *argv[])
{
  try
    {
      if (argc > 1)
	{
          logStream (&std::cout);
	}

      runTest ();

      return 0;
    }

  catch (std::runtime_error const &err)
    {
      std::cerr << err.what() << "\n";
    }

  return 1;
}

namespace
{
  void runTest ()
  {
    World world (std::cin);

    Searcher *searchPtr (&world.searcher());

    SpanningTree tree (searchPtr->treeSize(), 0);

    int nearest = -1;

    searchPtr->search (tree, Searcher::AnyBaseLimiter (&nearest));

    logStream() << world.board();

    if (nearest == -1)
      {
	std::cout << "No bases on field\n";
      }

    else
      {
	Path path (tree.pathFromRoot (nearest));

	int start = 0;

	while (start != nearest)
	  {
	    int next = path.next (start, nearest);
	    std::cout << next << " ";

	    start = next;
	  }

	while (start != 0)
	  {
	    int next = path.next (start, 0);
	    std::cout << next << " ";

	    start = next;
	  }

	std::cout << std::endl;
      }
  }
}
