// -*- mode:c++ -*-
//
// Module testworld.cc
//
// Copyright (c) 2002 Raoul M. Gough
//
// History
// =======
// 2002/ 8/31	rmg	File creation
//
// $Id: testworld.cc,v 1.2 2002/08/31 18:12:41 raoul Exp $
//

#include "World.hh"
#include "LogStream.hh"
#include "ServerConnection.hh"

#include <stdexcept>

namespace
{
  void runTest (std::istream &);
}

int main (int argc, char *argv[])
{
  try
    {
      if (argc == 1)
	{
	  std::cout
	    << "Reading from standard input."
	    " To connect to a server, use: testworld host port\n";

	  runTest (std::cin);
	}

      else
	{
	  std::cout << "Attempting server connection\n";

	  ServerConnection conn (argc, argv);

	  std::cout << "Connected\n";

	  *conn.outPtr() << "Player" << std::endl;

	  runTest (*conn.inPtr());
	}
    }

  catch (std::runtime_error const &err)
    {
      std::cerr << "Caught error: " << err.what() << std::endl;
    }
}

namespace
{
  void runTest (std::istream &strm)
  {
    logStream (&std::cerr);

    World world (strm);

    while (strm)
      {
	world.update (strm);
      }
  }
}
