// -*- mode:c++ -*-
//
// Header file ucv.hh
//
// Copyright (c) 2002 Raoul M. Gough
//
// This template represents uncertain values.
//
// History
// =======
// 2002/ 8/31	rmg	File creation
//
// $Id: ucv.hh,v 1.1 2002/08/31 13:04:00 raoul Exp $
//

#ifndef ucv_rmg_20020831_included
#define ucv_rmg_20020831_included

#include <stdexcept>
#include <string>
#include <iostream>

enum Certainty {
  full
  , partial
  , none
};

template<typename T>
class ucv
{
  Certainty mCertainty;
  T mGuess;
  T mUpRange;   // max ("real" value - mGuess)
  T mDownRange; // max (mGuess - "real" value)

public:
  ucv ()
    : mCertainty (none), mGuess(), mUpRange (), mDownRange () { }

  ucv (T const &v)
    : mCertainty (full), mGuess(v), mUpRange (0), mDownRange (0) { }

  ucv (T const &v, T const &up, T const &down)
    : mCertainty (partial), mGuess(v), mUpRange (up), mDownRange (down)
  {
    if ((mUpRange == 0) && (mDownRange == 0))
      {
	mCertainty = full;
      }
  }

  inline Certainty certainty () const { return mCertainty; }

  inline T const &guess () const
  {
    if (mCertainty == none)
      {
	throw std::runtime_error ("Can't guess");
      }

    return mGuess;
  };

  inline T const &max () const { return guess() + mUpRange; }
  inline T const &min () const { return guess() - mDownRange; }
  inline T const &uprange () const { return mUpRange; }
  inline T const &downrange () const { return mDownRange; }

  void guess (T const &v)
  {
    mGuess = v;
    mCertainty = full;
    mUpRange = 0;
    mDownRange = 0;
  }

  void guess (T const &v, T const &up, T const &down)
  {
    mGuess = v;

    if ((up == 0) && (down == 0))
      {
	mCertainty = full;
      }

    else
      {
	mCertainty = partial;
      }

    mUpRange = up;
    mDownRange = down;
  }

  void uprange (T const &v)
  {
    mUpRange = v;

    if (mCertainty == full)
      {
	mCertainty = partial;
      }
  }

  void downrange (T const &v)
  {
    mDownRange = v;

    if (mCertainty == full)
      {
	mCertainty = partial;
      }
  }

  ucv &operator+= (T &v) { guess (guess() + v, mUpRange, mDownRange); }
  ucv &operator-= (T &v) { guess (guess() - v, mUpRange, mDownRange); }
};

template<typename T>
ostream &operator<< (ostream &strm, ucv<T> const &v)
{
  if (v.certainty() == none)
    {
      strm << "unknown";
    }

  else
    {
      strm << v.guess();

      if (v.certainty() != full)
	{
	  strm
	    << "+" << v.uprange()
	    << "-" << v.downrange();
	}
    }

  return strm;
}

#endif // ucv_rmg_20020831_included
